﻿using System;
using System.Data;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.ServiceModel.Syndication;
using System.Collections.Generic;
using System.IO;
using System.Reflection;

namespace RssServiceApp
{
	[ServiceContract]
	interface IRssService
	{
		[OperationContract]
		[WebGet]
		Rss20FeedFormatter GetProducts();
	}

	class RssService : IRssService
	{
		public Rss20FeedFormatter GetProducts()
		{
			SyndicationFeed feed = new SyndicationFeed(
				"Northwind Products (Title)",
				"Northwind Products List (Description)",
				new Uri("http://www.northwindtrader.com/product/showall.aspx"));
			SyndicationPerson person = new SyndicationPerson("supprot@northwindtrader.com");
			SyndicationCategory category = new SyndicationCategory("All Products");
			feed.Authors.Add(person);
			feed.Categories.Add(category);

			DataSet ds = GetProductDataSet();
			List<SyndicationItem> productList = new List<SyndicationItem>();
			foreach (DataRow row in ds.Tables[0].Rows) {
				string title = row["ProductName"].ToString();
				string desc = String.Format(
					"<font bold='true'>ProductName : {0}</font><br/>" +
					"Quantity per Unit : {1}<br/>" +
					"Unit Price : {2}<br/>",
					row["ProductName"], row["QuantityPerUnit"], row["UnitPrice"]);
				string url = string.Format(
					"http://www.northwindtrader.com/product/showproduct.aspx?id={0}",
					row["ProductId"]);
				SyndicationItem item = new SyndicationItem(title, desc, new Uri(url));
				productList.Add(item);
			}
			feed.Items = productList;
			Console.WriteLine("RSS Feed has created......");
			return new Rss20FeedFormatter(feed);
		}

		private DataSet GetProductDataSet()
		{
			DataSet ds = new DataSet();
			Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream("RssServiceApp.Products.xml");
			ds.ReadXml(stream, XmlReadMode.ReadSchema);
			stream.Close();
			return ds;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("RSS Feed Service App......");

			WebServiceHost host = new WebServiceHost(typeof(RssService),
				new Uri("http://localhost/wcf/example/syndicate/rss"));
			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
