﻿using System;
using System.Net;
using System.Threading;
using System.IO;

namespace SimpleHttpRouter
{
	class HttpMessageRouter
	{
		private HttpListener _Listener;
		private Thread _WorkerThread;
		private string _ForwardUri;
		private string _ListenUri;

		// 생성자
		public HttpMessageRouter(string listenUri, string forwardUri)
		{
			// 리스닝할 URI 설정
			_ListenUri = listenUri;
			// 포워드할 URI 설정
			_ForwardUri = forwardUri;
			// 작업 쓰레드 설정
			_WorkerThread = new Thread(new ThreadStart(WorkerThreadMain));
		}

		// HTTP 리스닝을 시작한다.
		public void Open()
		{
			// HttpListener 초기화
			_Listener = new HttpListener();
			_Listener.Prefixes.Add(_ListenUri);
			_Listener.AuthenticationSchemes = AuthenticationSchemes.Anonymous;
			_Listener.Start();

			_WorkerThread.Start();
		}

		// HTTP 리스닝을 중단한다.
		public void Close()
		{
			// 작업 쓰레드를 Abort 하여 쓰레드를 종료시킨다.
			// NOTE : 이렇게 쓰레드를 강제로 종료하는 것은 알흠답지 않은 코드이다.
			//        동기화 객체 등을 사용하여 쓰레드가 알흠답게 종료하도록 해야만 한다.
			//        대략 귀찮으므로 생략... -_-;
			_Listener.Stop();
			_Listener.Close();

			if (_WorkerThread.ThreadState == ThreadState.Running) {
				_WorkerThread.Abort();
				_WorkerThread.Join();
			}
		}

		// 스트림을 복사 한다.
		private void CopyStream(Stream srcStream, Stream destStream)
		{
			byte[] buffer = new byte[4096];
			int nRead;
			while (true) {
				nRead = srcStream.Read(buffer, 0, buffer.Length);
				if (nRead == 0) {
					break;
				}
				destStream.Write(buffer, 0, nRead);
			}
		}

		public void WorkerThreadMain()
		{
			Random random = new Random();
			while (true) {
				HttpListenerContext ctx = _Listener.GetContext();
				int randomValue = random.Next();
				if (randomValue % 4 == 2) {
					ThreadPool.QueueUserWorkItem(new WaitCallback(DropMessage), ctx);
				}
				else {
					ThreadPool.QueueUserWorkItem(new WaitCallback(RouteMessage), ctx);
				}
			}
		}

		private void RouteMessage(object arg)
		{
			HttpListenerContext ctx = (HttpListenerContext)arg;
			Stream srcStream, destStream;

			HttpWebRequest req = (HttpWebRequest)WebRequest.Create(_ForwardUri);
			foreach (string headerName in ctx.Request.Headers.Keys) {
				try {
					req.Headers[headerName] = ctx.Request.Headers[headerName];
				}
				catch { }
			}
			req.Method = ctx.Request.HttpMethod;
			req.ContentLength = ctx.Request.ContentLength64;
			req.ContentType = ctx.Request.ContentType;
			destStream = req.GetRequestStream();
			srcStream = ctx.Request.InputStream;
			CopyStream(srcStream, destStream);

			HttpWebResponse res = (HttpWebResponse)req.GetResponse();
			ctx.Response.StatusCode = (int)res.StatusCode;
			foreach (string headerName in res.Headers.Keys) {
				try {
					ctx.Response.Headers[headerName] = res.Headers[headerName];
				}
				catch { }
			}
			destStream = ctx.Response.OutputStream;
			srcStream = res.GetResponseStream();
			CopyStream(srcStream, destStream);

			ctx.Response.Close();
		}

		private static void DropMessage(object arg)
		{
			HttpListenerContext ctx = (HttpListenerContext)arg;
			ctx.Response.StatusCode = 500;
			ctx.Response.ContentType = "application/soap+xml; charset=utf-8";
			string responseString = "<HTML><BODY>Message Is Dropped</BODY></HTML>";
			byte[] buffer = System.Text.Encoding.UTF8.GetBytes(responseString);
			// Get a response stream and write the response to it.
			ctx.Response.ContentLength64 = buffer.Length;
			System.IO.Stream output = ctx.Response.OutputStream;
			output.Write(buffer, 0, buffer.Length);
			// You must close the output stream.
			output.Close();
			ctx.Response.Close();
			Console.WriteLine("Message dropped....");
		}
	}
}
