﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Channels;
using System.Net.Security;

namespace SecureServiceApp
{
	[ServiceContract]
	interface ISecureService
	{
		[OperationContract]
		void SecureMethod(string arg);
	}

	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class SecureService : ISecureService
	{
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ServiceSecurityContext ctx = OperationContext.Current.ServiceSecurityContext;
			if (ctx.IsAnonymous) {
				Console.WriteLine("  Anonymous client");
			}
			else {
				Console.WriteLine("  User = {0}", ctx.PrimaryIdentity.Name);
				Console.WriteLine("  AuthenticateType = {0}", ctx.PrimaryIdentity.AuthenticationType);
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			// 다음 메쏘드들 중 하나만을 호출해야 하며
			// 메쏘드 호출을 변경하면 클라이언트의 서비스 참조를 업데이트 해야 한다.
			//TcpTransportSecurityWithAnonymousCredential();
			//TcpTransportSecurityWithWindowsCredential();
			TcpTransportSecurityWithCertficateCredential();
		}

		private static void TcpTransportSecurityWithAnonymousCredential()
		{
			Console.WriteLine("TCP Transport Security Test Service with Anonymous Credential...");

			NetTcpBinding binding = new NetTcpBinding();
			binding.Security.Mode = SecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType = TcpClientCredentialType.None;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("net.tcp://localhost:8000/wcf/example/security/tcp"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");
			host.Credentials.ServiceCertificate.SetCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			host.Description.Behaviors.Add(behavior);
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexTcpBinding(), "mex");

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}

		private static void TcpTransportSecurityWithWindowsCredential()
		{
			Console.WriteLine("TCP Transport Security Test Service with Windows Credential...");

			NetTcpBinding binding = new NetTcpBinding();
			binding.Security.Mode = SecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType = TcpClientCredentialType.Windows;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("net.tcp://localhost:8000/wcf/example/security/tcp"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			host.Description.Behaviors.Add(behavior);
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexTcpBinding(), "mex");

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}

		private static void TcpTransportSecurityWithCertficateCredential()
		{
			Console.WriteLine("TCP Transport Security Test Service with Certificate Credential...");

			NetTcpBinding binding = new NetTcpBinding();
			binding.Security.Mode = SecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType = TcpClientCredentialType.Certificate;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("net.tcp://localhost:8000/wcf/example/security/tcp"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");
			host.Credentials.ServiceCertificate.SetCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			host.Description.Behaviors.Add(behavior);
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexTcpBinding(), "mex");

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
