﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace RawChannelProgramming
{
	class TestApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Raw Channel Programming Sample......");

			// 바인딩 및 서비스 주소
			BasicHttpBinding binding = new BasicHttpBinding();
			string serviceAddress = "http://localhost/wcf/example/channel";
			// 바인딩으로부터 채널 리스너 스택 생성 및 오픈
			IChannelListener<IReplyChannel> listenserStack = 
				binding.BuildChannelListener<IReplyChannel>(
					new Uri(serviceAddress), new BindingParameterCollection());
			listenserStack.Open();
			// 채널 리스너 스택으로 부터 채널 스택 생성
			IReplyChannel replyChannel = listenserStack.AcceptChannel();
			replyChannel.Open();
			// 메시지 수신 시작
			replyChannel.BeginReceiveRequest(AsyncHandler, replyChannel);

			// 클라이언트 측 채널 팩토리 스택 생성 및 오픈
			IChannelFactory<IRequestChannel> channelFactoryStack = 
				binding.BuildChannelFactory<IRequestChannel>(
					new BindingParameterCollection());
			channelFactoryStack.Open();
			// 채널 스택 생성
			IRequestChannel requestChannel = channelFactoryStack.CreateChannel(
				new EndpointAddress(serviceAddress));
			requestChannel.Open();
			// 전송할 메시지 구성
			Message requestMessage = Message.CreateMessage(
				MessageVersion.Soap11, "urn:Test", "Hello");
			// 전송 !
			Message replyMessage = requestChannel.Request(requestMessage);
			// 결과 표시
			string content = replyMessage.GetBody<string>();
			Console.WriteLine("Reply from service : {0}", content);
		}

		static void AsyncHandler(IAsyncResult ar)
		{
			IReplyChannel replyChannel = (IReplyChannel)ar.AsyncState;
			RequestContext requestContext = replyChannel.EndReceiveRequest(ar);
			Message requestMessage = requestContext.RequestMessage;
			string content = requestMessage.GetBody<string>();
			Console.WriteLine("The service invoked ! message content = {0}", content);
			Message replyMessage = Message.CreateMessage(
				MessageVersion.Soap11, "urn:TestReply", "Result");
			requestContext.Reply(replyMessage);

			System.ServiceModel.Dispatcher.ChannelDispatcher dispatcher;
		}
	}
}
