﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace DataContractVersioningService
{
	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	class BookInfo
	{
		[DataMember]
		public int Id;
		[DataMember]
		public string Title;
		[DataMember]
		public AuthorInfo AuthorInfo;
		[DataMember(IsRequired = true)]
		public string Isbn;
	}

	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	class AuthorInfo : IExtensibleDataObject
	{
		[DataMember]
		public string FullName;
		// Non-required 멤버는 데이터 계약에서 제외되어도 안전하다.
		// 레거시 클라이언트는 단순히 디폴트 값으로 null을 받게 된다.
		//[DataMember]
		public string Address;
		[DataMember]
		public string Email;

		private ExtensionDataObject _ExtensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get { return _ExtensionDataObject; }
			set { _ExtensionDataObject = value; }
		}
	}

	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		BookInfo AddBook(BookInfo book);
		[OperationContract]
		BookInfo GetBookInfo(string isbn);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService : IBookStore
	{
		private static Dictionary<string, BookInfo> _Books = new Dictionary<string, BookInfo>();

		static BookStoreService()
		{
			AuthorInfo author;
			BookInfo book;

			author = new AuthorInfo();
			author.FullName = "김정복";
			author.Address = "서울시 정복구 정복동 정복아파트 1동 1호";
			author.Email = "nobody@noname.com";

			book = new BookInfo();
			book.Id = 1;
			book.AuthorInfo = author;
			book.Title = "WCF 1일만에 정복";
			book.Isbn = "000-001";
			_Books.Add(book.Isbn, book);

			book = new BookInfo();
			book.Id = 2;
			book.AuthorInfo = author;
			book.Title = "ASP.NET 웹 서비스 1일만에 정복";
			book.Isbn = "000-002";
			_Books.Add(book.Isbn, book);
		}

		public BookInfo AddBook(BookInfo book)
		{
			if (_Books.ContainsKey(book.Isbn)) {
				_Books[book.Isbn] = book;
			}
			else {
				// Id를 설정하여 반환
				book.Id = _Books.Count + 1;
				_Books.Add(book.Isbn, book);
			}
			return book;
		}

		public BookInfo GetBookInfo(string isbn)
		{
			if (_Books.ContainsKey(isbn) == true) {
				return _Books[isbn];
			}
			else {
				return null;
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Data Contract Versioning Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IBookStore),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
