﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace LegacyClient
{
	//
	// 레거시 클라이언트를 에뮬레이트하기 위해 계약을 직접 선언하였다.
	//
	#region "서비스 계약 및 데이터 계약 선언"

	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	class BookInfo
	{
		[DataMember]
		public int Id;
		[DataMember]
		public string Title;
		[DataMember]
		public AuthorInfo AuthorInfo;
		[DataMember(IsRequired = true)]
		public string Isbn;
	}

	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	class AuthorInfo : IExtensibleDataObject
	{
		[DataMember]
		public string FullName;
		[DataMember]
		public string Address;
		[DataMember]
		public string Email;

		private ExtensionDataObject _ExtensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get { return _ExtensionDataObject; }
			set { _ExtensionDataObject = value; }
		}
	}

	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		BookInfo AddBook(BookInfo book);
		[OperationContract]
		BookInfo GetBookInfo(string isbn);
	}
	#endregion

	class LegacyClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Composite Data Contract Client Example ......");

			// 자동 생성 프록시를 사용하지 않으므로 직접 채널을 생성한다.
			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(IBookStore)));
			ChannelFactory<IBookStore> factory = new ChannelFactory<IBookStore>(
				new BasicHttpBinding(),
				"http://localhost/wcf/example/bookstoreservice");

			AuthorInfo author;
			BookInfo book;

			author = new AuthorInfo();
			author.FullName = "홍길동";
			author.Address = "서울시 길동구 길동 123-4";
			author.Email = "mail@noname.com";

			book = new BookInfo();
			book.Id = -1;
			book.AuthorInfo = author;
			book.Title = "변신술에 관한 고찰 part II";
			book.Isbn = "000-010";

			IBookStore proxy = factory.CreateChannel();
			using (proxy as IDisposable) {
				BookInfo info = proxy.GetBookInfo("000-001");
				Console.WriteLine("Book Information: Id = {0}\n  Title={0}\n  ISBN={1}",
					info.Id, info.Title, info.Isbn);
				Console.WriteLine("  Author={0}\n      Email={1}\n      Address={2}",
					info.AuthorInfo.FullName, info.AuthorInfo.Email, info.AuthorInfo.Address);

				info = proxy.AddBook(book);
				Console.WriteLine("Added Book Information: Id = {0}\n  Title={0}\n  ISBN={1}",
					info.Id, info.Title, info.Isbn);
				Console.WriteLine("  Author={0}\n      Email={1}\n      Address={2}",
					info.AuthorInfo.FullName, info.AuthorInfo.Email, info.AuthorInfo.Address);
			}
		}
	}
}
