﻿using System;
using System.Runtime.Serialization;
using System.IO;

namespace NetDataContractSerializerSample
{
	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	class BookInfo
	{
		[DataMember]
		public string Title;
		[DataMember(IsRequired = true)]
		public readonly string Isbn;
		[DataMember]
		public string Author;
		[DataMember]
		public int Price;
		// 이 필드는 직렬화에 해당 사항이 없다.
		public double MarginRate;

		public BookInfo(string title, string isbn, string author, int price)
		{
			Title = title;
			Isbn = isbn;
			Author = author;
			Price = price;
			MarginRate = 0.25;
		}

		public override string ToString()
		{
			return String.Format("Isbn={0}, Title = {1}\n   Author={2}, Price={3} Won, Margin={4:#0}%",
				Isbn, Title, Author, Price, MarginRate * 100);
		}
	}

	class SerializationTest
	{
		static void Main(string[] args)
		{
			Console.WriteLine("DataContractSerializer Sample...\n");

			BookInfo book = new BookInfo("WCF 그것을 알려주마", "000-001", "김알림", 30000);
			Console.WriteLine("Before serialize...");
			Console.WriteLine(book);
			Serialize(book);
			Serialize2(book);
			Console.WriteLine();

			book = Deserialize();
			Console.WriteLine("After serialize...");
			Console.WriteLine(book);
		}

		private static void Serialize(BookInfo book)
		{
			NetDataContractSerializer serializer = new NetDataContractSerializer();
			FileStream stream = new FileStream("BookInfo.xml", FileMode.Create);
			serializer.WriteObject(stream, book);
			stream.Close();
		}

		private static void Serialize2(BookInfo book)
		{
			// 다음과 같이 루트 요소를 재지정 할 수도 있다.
			NetDataContractSerializer serializer = new NetDataContractSerializer(
				"MyRootElement", "http://noname.com/customns");
			FileStream stream = new FileStream("BookInfo2.xml", FileMode.Create);
			serializer.WriteObject(stream, book);
			stream.Close();
		}

		private static BookInfo Deserialize()
		{
			NetDataContractSerializer serializer = new NetDataContractSerializer();
			FileStream stream = new FileStream("BookInfo.xml", FileMode.Open);
			BookInfo book = (BookInfo)serializer.ReadObject(stream);
			stream.Close();
			return book;
		}
	}
}
