﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace SerializableDataContractService
{
	[Serializable]
	class BookInfo
	{
		[OptionalField]
		private string Title;
		public string TitleProp
		{
			get { return Title; }
			set { Title = value; }
		}
		public string Isbn;
		[OptionalField]
		public string Author;
		[OptionalField]
		public int Price;
		[NonSerialized]
		public double MarginRate;

		public BookInfo(string title, string isbn, string author, int price)
		{
			Title = title;
			Isbn = isbn;
			Author = author;
			Price = price;
			MarginRate = 0.25;
		}
	}

	// 서비스 계약 선언
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		BookInfo GetBookInfo(string isbn);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService : IBookStore
	{
		private static Dictionary<string, BookInfo> _Books = new Dictionary<string, BookInfo>();

		static BookStoreService()
		{
			BookInfo info;

			info = new BookInfo("WCF 10일만에 정복", "000-001", "김정복", 40000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("10일동안 하면 개똥이 만큼 WCF 한다", "000-002", "김개똥", 42000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Programming", "000-003", "Mr. Programming", 53000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Programming WCF", "000-004", "Mrs. Programming", 37000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Learning WCF", "000-005", "Unknown", 18000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Step by Step", "000-006", "김철수", 20000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo(null, "000-007", null, 20000);
			_Books.Add(info.Isbn, info);
		}

		public BookInfo GetBookInfo(string isbn)
		{
			if (_Books.ContainsKey(isbn) == true) {
				return _Books[isbn];
			}
			else {
				return null;
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Serializable Data Contract Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IBookStore),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
