﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

using ServiceImplementation;

namespace NetDataContractSerializerService
{
	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Using NetDataContractSerializer Example...");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IBookStore),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();

			// NetDataContractOperationBehavior 가 설정된 메쏘드들을 표시한다.
			// ServiceHost.Open에 의해 IxxxxBehavior.ApplyDispatchBehavior 메쏘드가 호출되므로
			// 서비스 Open 후에 검사를 해야만 한다.
			foreach (OperationDescription od in host.Description.Endpoints[0].Contract.Operations) {
				NetDataContractSerializerOperationBehavior behavior2 =
					od.Behaviors.Find<NetDataContractSerializerOperationBehavior>();
				if (behavior2 != null) {
					Console.WriteLine("Set NetDataContractSerializerOperationBehavior : {0}",
						od.SyncMethod.Name);
				}
			}

			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
