﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace MaxConnectionSettingService
{
	[ServiceContract(Namespace = "http://noname.com/myservice")]
	interface IMyContract
	{
		[OperationContract]
		string SayHello();
	}

	class MyService : IMyContract
	{
		public string SayHello()
		{
			Console.WriteLine("SayHello() invoked...");
			return "Hello NetTcpBinding World !";
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Setting MaxConnections of NetTcpBinding Sample Service......");

			ServiceHost host = new ServiceHost(typeof(MyService),
				new Uri("net.tcp://localhost:8000/myservice"));

			// TCP나 Named pipe와 같은 연결 지향 트랜스포트를 사용하면 서비스는 클라이언트와의
			// 통신이 완료된 이후일지라도 연결을 곧바로 종료하지 않고 유지한다.
			// 이러한 연결 유지(keep alive)는 추후에 클라이언트가 서비스에 다시 접속할 때
			// 상대적으로 비싼 비용의 TCP 연결을 다시 수행하지 않아도 되도록 해준다.
			// NetTcpBinding의 MaxConnections 속성 값은 바로 서비스 측에서 지속적으로 유지
			// 할 수 있는 Tcp 연결의 최대값을 설정하는데 사용된다. 
			// 속성 이름에 현혹되지 말자.
			NetTcpBinding binding = new NetTcpBinding();
			binding.MaxConnections = 3;
			binding.Security.Mode = SecurityMode.None;
			Console.WriteLine("NetTcpBinding.MaxConnections = {0}", binding.MaxConnections);

			host.AddServiceEndpoint(typeof(IMyContract), binding, "");

			// 서비스 리스너 구동 및 서비스 수행
			host.Open();
			Console.WriteLine("Press Any Key to Stop Service......");
			Console.ReadKey(true);

			// 서비스 종료
			host.Close();
		}
	}
}
