﻿using System;
using System.Runtime.Serialization;
using System.ServiceModel.Channels;
using System.Xml;
using System.IO;
using System.Text;

namespace ProgrammaticMessageEncoding
{
	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	class BookInfo
	{
		[DataMember]
		public string Title;
		[DataMember]
		public string Isbn;
		[DataMember]
		public string Author;
		[DataMember]
		public int Price = 0x77ff77ff;
		[DataMember]
		public byte[] Content;

		public override string ToString()
		{
			return String.Format("{0},{1}", Author, Title);
		}
	}

	class Program
	{
		static void Main(string[] args)
		{
			BookInfo info = new BookInfo();
			info.Title = "WCF TECHNOLOGY BOOK";
			info.Isbn = "1111-2222-3333";
			info.Author = "WRITER";
			info.Content = new byte[32];
			for (int i = 0; i < info.Content.Length; i++) {
				info.Content[i] = (byte)(i % 26 + 0x41);
			}

			CreateTextMessage(info);
			CreateMtomMessage(info);
			CreateBinaryMessage(info);
		}

		// Text 인코딩을 사용하여 SOAP 메시지를 구성해 본다.
		private static void CreateTextMessage(BookInfo info)
		{
			Console.Write("Creating text message......");

			FileStream stream = new FileStream("bookinfo.xml", FileMode.Create);
			XmlDictionaryWriter writer = XmlDictionaryWriter.CreateTextWriter(stream);
			DataContractSerializer serializer = new DataContractSerializer(typeof(BookInfo));
			Message msg = Message.CreateMessage(MessageVersion.Soap12WSAddressing10,
				"http://noname.com/encoding", info, serializer);
			msg.WriteMessage(writer);
			msg.Close();
			writer.Close();

			Console.WriteLine(" Done.");
		}

		// Mtom 인코딩을 사용하여 SOAP 메시지를 구성해 본다.
		private static void CreateMtomMessage(BookInfo info)
		{
			Console.Write("Creating MTOM message......");

			FileStream stream = new FileStream("bookinfo.txt", FileMode.Create);
			XmlDictionaryWriter writer = XmlDictionaryWriter.CreateMtomWriter(stream,
				Encoding.UTF8, 1024*1024, "application/soap+xml");
			DataContractSerializer serializer = new DataContractSerializer(typeof(BookInfo));
			Message msg = Message.CreateMessage(MessageVersion.Soap12WSAddressing10,
				"http://noname.com/encoding", info, serializer);
			msg.WriteMessage(writer);
			msg.Close();
			writer.Close();

			Console.WriteLine(" Done.");
		}

		// Mtom 인코딩을 사용하여 SOAP 메시지를 구성해 본다.
		private static void CreateBinaryMessage(BookInfo info)
		{
			Console.Write("Creating binary message......");

			FileStream stream = new FileStream("bookinfo.bin", FileMode.Create);
			XmlDictionaryWriter writer = XmlDictionaryWriter.CreateBinaryWriter(stream);
			DataContractSerializer serializer = new DataContractSerializer(typeof(BookInfo));
			Message msg = Message.CreateMessage(MessageVersion.Soap12WSAddressing10,
				"http://noname.com/encoding", info, serializer);
			msg.WriteMessage(writer);
			msg.Close();
			writer.Close();

			Console.WriteLine(" Done.");
		}
	}
}
