﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace SslBasicHttpBindingService
{
	// 간단한 Echo 서비스 계약
	[ServiceContract]
	interface IMyContract
	{
		[OperationContract]
		string Echo(string arg);
	}

	// 서비스 구현 클래스
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class MyService : IMyContract
	{
		public string Echo(string arg)
		{
			// 말그대로 에코만... ^^
			return arg;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("BasicHttpBinding with SSL Sample Service......");

			// 서비스의 베이스 주소가 HTTPS 임에 유의한다.
			ServiceHost host = new ServiceHost(typeof(MyService),
				new Uri("https://localhost/wcf/example/myservice"));

			// 바인딩 설정
			// 트랜스포트 보안(SSL)을 사용하고 클라이언트는 익명 인증 사용
			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.None;

			// 서비스 종점 설정
			host.AddServiceEndpoint(typeof(IMyContract), binding, "");

			// WSDL에 대한 HTTP GET을 enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpsGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			// 서비스 리스너 구동 및 서비스 수행
			host.Open();
			Console.WriteLine("Press Any Key to Stop Service......");
			Console.ReadKey(true);

			// 서비스 종료
			host.Close();
		}
	}
}
