﻿using System;
using System.ServiceModel;
using System.Configuration;

using ClientAppUsingFactory.Service1;
using ClientAppUsingFactory.Service2;
using ClientAppUsingFactory.Service3;

namespace ClientAppUsingFactory
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Client example with Proxy Factory......");

			using (Contract1Client proxy = CreateHttpProxy<Contract1Client>("service1")) {
				proxy.Method1();
			}
			using (Contract1Client proxy = CreateTcpProxy<Contract1Client>("service1")) {
				proxy.Method1();
			}

			using (Contract2Client proxy = CreateHttpProxy<Contract2Client>("service2")) {
				proxy.Method2();
			}
			using (Contract2Client proxy = CreateTcpProxy<Contract2Client>("service2")) {
				proxy.Method2();
			}

			using (Contract3Client proxy = CreateHttpProxy<Contract3Client>("service3")) {
				proxy.Method3();
			}
			using (Contract3Client proxy = CreateTcpProxy<Contract3Client>("service3")) {
				proxy.Method3();
			}

		}

		static T CreateHttpProxy<T>(string relativeAddress)
		{
			string baseAddress = ConfigurationManager.AppSettings["HttpBaseAddress"];
			WSHttpBinding binding = new WSHttpBinding("HttpBindingConfig");
			EndpointAddress remoteAddress = 
				new EndpointAddress(baseAddress + relativeAddress);

			T proxy = (T) Activator.CreateInstance(typeof(T),
				new object[] { binding, remoteAddress });
			return proxy;
		}

		static T CreateTcpProxy<T>(string relativeAddress)
		{
			string baseAddress = ConfigurationManager.AppSettings["TcpBaseAddress"];
			NetTcpBinding binding = new NetTcpBinding("TcpBindingConfig");
			EndpointAddress remoteAddress = 
				new EndpointAddress(baseAddress + relativeAddress);

			T proxy = (T)Activator.CreateInstance(typeof(T),
				new object[] { binding, remoteAddress });
			return proxy;
		}
	}
}
