﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;

namespace ExceptionServiceApp
{
	// 사용자 정의 에러 핸들러 구현
	class MyErrorHandler : IErrorHandler
	{
		public bool HandleError(Exception error)
		{
			Console.WriteLine("Error [{0}]::{1}", error.GetType().Name, error.Message);
			return false;
		}

		public void ProvideFault(Exception error, MessageVersion version, ref Message fault)
		{
			if (fault == null) {
				// 발생한 예외를 FaultException으로 wrapping 한다.
				ExceptionDetail detail = new ExceptionDetail(error);
				FaultCode code = FaultCode.CreateReceiverFaultCode(
					new FaultCode("Server." + error.GetType().Name));
				FaultReason reason = new FaultReason(error.Message);
				FaultException<ExceptionDetail> faultException =
					new FaultException<ExceptionDetail>(detail, reason, code);
				MessageFault messageFault = faultException.CreateMessageFault();
				fault =
					Message.CreateMessage(version, messageFault, faultException.Action);
			}
		}
	}
}
