﻿using System;
using System.ServiceModel;

namespace PerSessionServiceApp
{
	// 서비스 계약 선언
	[ServiceContract(SessionMode = SessionMode.Required)]
	interface IPerSession
	{
		[OperationContract]
		int TestMethod();

		[OperationContract]
		void Recycle();

		[OperationContract]
		void RecycleIf(bool recycle);
	}

	// PerSession 서비스 타입 구현
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
	class PerSessionService : IPerSession, IDisposable
	{
		private int _CallCount;

		public PerSessionService()
		{
			Console.WriteLine(".ctor()   : Id={0}, Session={1}",
				this.GetHashCode(), OperationContext.Current.SessionId);
			_CallCount = 0;
		}

		public void Dispose()
		{
			Console.Write("Dispose() : Id={0}, ", this.GetHashCode());
			if (OperationContext.Current == null) {
				Console.WriteLine("OperationContext.Current == null");
			}
			else {
				Console.WriteLine("Session={1}",
					this.GetHashCode(), OperationContext.Current.SessionId);
			}
		}

		public int TestMethod()
		{
			_CallCount++;
			Console.WriteLine("*** TestMethod() : CallCount={0}", _CallCount);
			return _CallCount;
		}

		[OperationBehavior(ReleaseInstanceMode = ReleaseInstanceMode.AfterCall)]
		public void Recycle()
		{
			Console.WriteLine("*** Recycling service instance");
		}

		public void RecycleIf(bool recycle)
		{
			Console.WriteLine("*** Recycling if the variable recycle is true");
			if (recycle) {
				OperationContext.Current.InstanceContext.ReleaseServiceInstance();
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("ReleaseInstanceMode Test Service Sample......");

			// 서비스 호스트와 종점 설정은 configuration을 사용한다.
			ServiceHost host = new ServiceHost(typeof(PerSessionService));
			host.Open();
			Console.WriteLine("Press any key to stop the service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
