﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;

namespace SessionfulServiceApp
{
	// 서비스 계약 선언
	[ServiceContract(SessionMode=SessionMode.Required)]
	interface ISessionful
	{
		[OperationContract]
		int TestMethod1();
		[OperationContract]
		int TestMethod2();
	}

	// 서비스 타입 구현
	class SessionfulService : ISessionful, IDisposable
	{
		private int _CallCount;

		public SessionfulService()
		{
			Console.WriteLine("SessionfulService instance created...");
			_CallCount = 0;
		}

		public void Dispose()
		{
			Console.WriteLine("SessionfulService instance disposed...");
		}

		public int TestMethod1()
		{
			_CallCount++;
			Console.WriteLine("TestMethod1(): call count = {0}", _CallCount);
			return _CallCount;
		}

		public int TestMethod2()
		{
			_CallCount++;
			Console.WriteLine("TestMethod2(): call count = {0}", _CallCount);
			return _CallCount;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Sessionful Service Basic Test Sample......");

			// 서비스 호스트
			ServiceHost host = new ServiceHost(typeof(SessionfulService),
				new Uri("http://localhost/wcf/example/sessionfulservice"),
				new Uri("net.tcp://localhost/wcf/example/sessionfulservice"));
			// NetTcpBinding 설정
			NetTcpBinding netTcpBinding = new NetTcpBinding();
			netTcpBinding.Security.Mode = SecurityMode.None;
			// WSHttpBinding 설정
			WSHttpBinding wsHttpBinding = new WSHttpBinding();
			wsHttpBinding.Security.Mode = SecurityMode.None;
			wsHttpBinding.ReliableSession.Enabled = true;
			// 메타데이터 HTTP GET 활성화
			ServiceMetadataBehavior behavior =
				host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				host.Description.Behaviors.Add(behavior);
			}
			behavior.HttpGetEnabled = true;
			// 서비스 종점 설정
			host.AddServiceEndpoint(typeof(ISessionful), netTcpBinding, "");
			host.AddServiceEndpoint(typeof(ISessionful), wsHttpBinding, "");
			// 서비스 구동
			host.Open();
			Console.WriteLine("Press any key to stop the service...");
			Console.ReadKey(true);
			// 서비스 종료
			host.Close();
		}
	}
}
