﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Diagnostics;
using TestClientApp.PerCall;
using TestClientApp.PerSession;
using TestClientApp.Singleton;

namespace TestClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Single Concurrency Mode Test Client......");

			// 세션을 사용하지 않는 바인딩 사용
			//TestPerCallService("BasicHttpBinding_IPerCallService");
			//TestPerCallService2("BasicHttpBinding_IPerCallService");
			// 세션을 사용하는 바인딩 사용
			//TestPerCallService("NetTcpBinding_IPerCallService");
			//TestPerCallService2("NetTcpBinding_IPerCallService");

			// 세션을 사용하지 않는 바인딩 사용 (PerCall과 동일하게 작동)
			//TestPerSessionService("BasicHttpBinding_IPerSessionService");
			//TestPerSessionService2("BasicHttpBinding_IPerSessionService");
			// 세션을 사용하는 바인딩 사용
			//TestPerSessionService("NetTcpBinding_IPerSessionService");
			//TestPerSessionService2("NetTcpBinding_IPerSessionService");

			// 세션을 사용하지 않는 바인딩 사용
			TestSingletonService("BasicHttpBinding_ISingletonService");
			//TestSingletonService2("BasicHttpBinding_ISingletonService");
			// 세션을 사용하는 바인딩 사용
			//TestSingletonService("NetTcpBinding_ISingletonService");
			//TestSingletonService2("NetTcpBinding_ISingletonService");
		}

		#region "PerCall 서비스 테스트"

		// 각 클라이언트 쓰레드에서 Proxy를 생성하도록 테스트.
		// 각 쓰레드에서 Proxy를 생성하므로 각 쓰레드는 서비스와 서로 다른
		// 세션(세션이 사용된다면)을 맺는다.
		private static void TestPerCallService(string endpointName)
		{
			ThreadStart func = delegate() {
				using (PerCallServiceClient proxy = new PerCallServiceClient(endpointName)) {
					Stopwatch clock = new Stopwatch();
					clock.Start();
					proxy.TestMethod(1000);
					clock.Stop();
					Console.WriteLine("[{0}] End of service call. time = {1} msec",
						Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);

				}
			};

			Console.WriteLine("PerCallService Test with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[3];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
		}

		// 각 클라이언트 쓰레드가 하나의 Proxy를 공유하도록 테스트.
		// BasicHttpBinding은 세션을 사용하지 않으므로 Proxy 공유 여부와 상관 없이 클라이언트의
		// 동시 호출은 서비스에서 동시에 처리된다.
		// NetTcpBinding을 사용하면 세션이 연결되고(Open 호출시) 인스턴스 문맥이 만들어진다.
		// 하지만 서비스 인스턴스는 아직 생성되지 않는다.
		// 각 쓰레드가 하나의 Proxy를 공유하므로 하나의 세션을 공유하며
		// 하나의 Instance Context 만을 사용한다.
		// PerCall 서비스이기 때문에 매 호출마다 새로운 인스턴스가 생성되지만
		// Single 병행성 모드가 Instance Context 에 대해 단일 쓰레드의 액세스만 허용하므로
		// 동시 호출은 순차적으로 처리 된다.
		private static void TestPerCallService2(string endpointName)
		{
			PerCallServiceClient proxy = new PerCallServiceClient(endpointName);
			// 세션을 맺기 위해 Open을 명시적으로 호출해야 한다.
			// 이 호출을 수행하지 않으면 서비스 호출(TestMethod 호출)에서 암시적으로 Open이
			// 수행되며 이 암시적 Open은 클라이언트 측에서 순차적으로 서비스를 호출하는
			// 결과를 낳게 된다. 따라서 원하는 테스트를 수행할 수 없다.
			proxy.Open();
			ThreadStart func = delegate() {
				Stopwatch clock = new Stopwatch();
				clock.Start();
				proxy.TestMethod(1000);
				clock.Stop();
				Console.WriteLine("[{0}] End of service call. time = {1} msec",
					Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);
			};

			Console.WriteLine("PerCallService Test #2 with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[3];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
			proxy.Close();
		}
		#endregion

		#region "PerSession 서비스 테스트"

		// 각 클라이언트 쓰레드에서 Proxy를 생성하도록 테스트.
		// 각 쓰레드에서 Proxy를 생성하므로 각 쓰레드는 서비스와 서로 다른
		// 세션(세션이 사용된다면)을 맺는다.
		private static void TestPerSessionService(string endpointName)
		{
			ThreadStart func = delegate() {
				using (PerSessionServiceClient proxy = new PerSessionServiceClient(endpointName)) {
					Stopwatch clock = new Stopwatch();
					clock.Start();
					proxy.TestMethod(1000);
					clock.Stop();
					Console.WriteLine("[{0}] End of service call. time = {1} msec",
						Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);

				}
			};

			Console.WriteLine("PerSessionService Test with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[3];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
		}

		// 각 클라이언트 쓰레드가 하나의 Proxy를 공유하도록 테스트.
		// BasicHttpBinding 과 같이 세션을 사용하지 않는 바인딩을 사용하면 PerCall 과 동일하게 작동한다.
		// NetTcpBinding 과 같이 세션을 사용하는 바인딩을 사용하면 하나의 세션에 하나의 인스턴스가 생성되며
		// 이 인스턴스는 유지된다.
		// 
		// 이 테스트에서는 각 쓰레드가 하나의 Proxy를 공유하므로 하나의 세션을 공유하며
		// 하나의 Instance Context 만을 사용한다(NetTcpBinding 사용시).
		// PerSession 서비스이기 때문에 세션에 연관된 하나의 인스턴스가 생성되고(Open 호출시)
		// Single 병행성 모드가 Instance Context 에 대해 단일 쓰레드의 액세스만 허용하므로
		// 동시 호출은 순차적으로 처리 된다.
		private static void TestPerSessionService2(string endpointName)
		{
			PerSessionServiceClient proxy = new PerSessionServiceClient(endpointName);
			// 세션을 맺기 위해 Open을 명시적으로 호출해야 한다.
			// 이 호출을 수행하지 않으면 서비스 호출(TestMethod 호출)에서 암시적으로 Open이
			// 수행되며 이 암시적 Open은 클라이언트 측에서 순차적으로 서비스를 호출하는
			// 결과를 낳게 된다. 따라서 원하는 테스트를 수행할 수 없다.
			proxy.Open();
			ThreadStart func = delegate() {
				Stopwatch clock = new Stopwatch();
				clock.Start();
				proxy.TestMethod(1000);
				clock.Stop();
				Console.WriteLine("[{0}] End of service call. time = {1} msec",
					Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);
			};

			Console.WriteLine("PerSessionService Test #2 with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[3];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
			proxy.Close();
		}
		#endregion

		#region "Singleton 서비스 테스트"

		// 각 클라이언트 쓰레드에서 Proxy를 생성하도록 테스트.
		// 각 쓰레드에서 Proxy를 생성하므로 각 쓰레드는 서비스와 서로 다른
		// 세션(세션이 사용된다면)을 맺는다.
		private static void TestSingletonService(string endpointName)
		{
			ThreadStart func = delegate() {
				using (SingletonServiceClient proxy = new SingletonServiceClient(endpointName)) {
					Stopwatch clock = new Stopwatch();
					clock.Start();
					proxy.TestMethod(1000);
					clock.Stop();
					Console.WriteLine("[{0}] End of service call. time = {1} msec",
						Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);

				}
			};

			Console.WriteLine("SingletonService Test with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[3];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
		}

		// 각 클라이언트 쓰레드가 하나의 Proxy를 공유하도록 테스트.
		// BasicHttpBinding 과 같이 세션을 사용하지 않는 바인딩을 사용하건
		// NetTcpBinding 과 같이 세션을 사용하는 바인딩을 사용하건
		// 하나의 서비스 인스턴스가 생성되며 이 인스턴스는 유지된다.
		// (세션이 사용되면 프록시마다 고유의 세션은 사용된다.)
		// 
		// 이 테스트에서는 각 쓰레드가 하나의 Proxy를 공유하므로 하나의 세션을 공유하며
		// 하나의 Instance Context 만을 사용한다(NetTcpBinding 사용시).
		// Singleton 서비스이기 때문에 세션에 연관된 하나의 인스턴스가 생성되고(Open 호출시)
		// Single 병행성 모드가 Instance Context 에 대해 단일 쓰레드의 액세스만 허용하므로
		// 동시 호출은 순차적으로 처리 된다.
		private static void TestSingletonService2(string endpointName)
		{
			SingletonServiceClient proxy = new SingletonServiceClient(endpointName);
			// 세션을 맺기 위해 Open을 명시적으로 호출해야 한다.
			// 이 호출을 수행하지 않으면 서비스 호출(TestMethod 호출)에서 암시적으로 Open이
			// 수행되며 이 암시적 Open은 클라이언트 측에서 순차적으로 서비스를 호출하는
			// 결과를 낳게 된다. 따라서 원하는 테스트를 수행할 수 없다.
			proxy.Open();
			ThreadStart func = delegate() {
				Stopwatch clock = new Stopwatch();
				clock.Start();
				proxy.TestMethod(1000);
				clock.Stop();
				Console.WriteLine("[{0}] End of service call. time = {1} msec",
					Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);
			};

			Console.WriteLine("SingletonService Test #2 with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[3];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
			proxy.Close();
		}
		#endregion
	}
}
