﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace TestServiceApp
{
	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Throttling Sample Service......");

			//StartPerCallService();
			StartPerSessionService();
			//StartSingletonService();
		}

		private static void ReportThrottingSetting(ServiceHost host)
		{
			ServiceThrottlingBehavior behavior = host.Description.Behaviors.Find<ServiceThrottlingBehavior>();
			// behavior가 설정되어 있지 않으면 새로이 behavior를 작성하여 디폴트 값을 표시한다.
			if (behavior == null) {
				behavior = new ServiceThrottlingBehavior();
			}
			Console.WriteLine("Throttling Setting :\n    MaxConcurrentSessions={0}\n" + 
				"    MaxConcurrentCalls={1}\n    MaxConcurrentInstances={2}",
				behavior.MaxConcurrentSessions, behavior.MaxConcurrentCalls, behavior.MaxConcurrentInstances);
		}

		private static void StartPerCallService()
		{
			ServiceHost host = new ServiceHost(typeof(PerCallService));
			// 코드에 의한 Throttling 설정
			ServiceThrottlingBehavior behavior = host.Description.Behaviors.Find<ServiceThrottlingBehavior>();
			if (behavior == null) {
				behavior = new ServiceThrottlingBehavior();
				behavior.MaxConcurrentSessions = 6;
				behavior.MaxConcurrentCalls = 3;
				behavior.MaxConcurrentInstances = 9;
				host.Description.Behaviors.Add(behavior);
			}
			host.Open();
			Console.WriteLine("PerCallService Host started......");
			ReportThrottingSetting(host);
			Console.ReadKey(false);
			host.Close();
		}

		private static void StartPerSessionService()
		{
			ServiceHost host = new ServiceHost(typeof(PerSessionService));
			host.Open();
			Console.WriteLine("PerSessionService Host started......");
			ReportThrottingSetting(host);
			Console.ReadKey(false);
			host.Close();
		}

		private static void StartSingletonService()
		{
			ServiceHost host = new ServiceHost(typeof(SingletonService));
			host.Open();
			Console.WriteLine("SingletonService Host started......");
			ReportThrottingSetting(host);
			Console.ReadKey(false);
			host.Close();
		}
	}
}
