﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.Threading;
using TestClientApp.LogService;

namespace TestClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			//ClientProxyTest();
			//ChannelProxyTest();
			ConcurrentAsyncCall();
		}

		private static void ClientProxyTest()
		{
			int logId;
			using (LogClient proxy = new LogClient("BasicHttpBinding_ILog")) {
				Console.WriteLine("Before LogMessage...");
				logId = proxy.LogMessage("Synchronous Call");
				Console.WriteLine("After LogMessage... LogId = {0}", logId);

				Console.WriteLine("Before BeginLogMessage...");
				IAsyncResult ar = proxy.BeginLogMessage("Asynchronous Call", null, null);
				Console.WriteLine("After BeginLogMessage...");

				Console.WriteLine("Do something...");

				Console.WriteLine("Before EndLogMessage...");
				logId = proxy.EndLogMessage(ar);
				Console.WriteLine("After EndLogMessage... LogId = {0}", logId);
			}
		}

		private static void ChannelProxyTest()
		{
			// 채널 팩터리를 통한 호출 예제 코드
			int logId;
			ChannelFactory<ILog> factory = new ChannelFactory<ILog>("BasicHttpBinding_ILog");
			ILog proxy = factory.CreateChannel();
			using (proxy as IDisposable) {
				Console.WriteLine("Before LogMessage...");
				logId = proxy.LogMessage("Synchronous Call");
				Console.WriteLine("After LogMessage... LogId = {0}", logId);

				Console.WriteLine("Before BeginLogMessage...");
				IAsyncResult ar = proxy.BeginLogMessage("Asynchronous Call", null, null);
				Console.WriteLine("After BeginLogMessage...");

				Console.WriteLine("Before EndLogMessage...");
				logId = proxy.EndLogMessage(ar);
				Console.WriteLine("After EndLogMessage... LogId = {0}", logId);
			}
		}

		private static void ConcurrentAsyncCall()
		{
			using (LogClient proxy = new LogClient("BasicHttpBinding_ILog")) {
				Console.WriteLine("Concurrent Asynchronous Call...");
				IAsyncResult ar1 = proxy.BeginLogMessage("Async Call #1", null, null);
				IAsyncResult ar2 = proxy.BeginLogMessage("Async Call #2", null, null);
				IAsyncResult ar3 = proxy.BeginLogMessage("Async Call #3", null, null);

				int logId1 = proxy.EndLogMessage(ar1);
				Console.WriteLine("Result of Async Call #1 : {0}", logId1);

				int logId2 = proxy.EndLogMessage(ar2);
				Console.WriteLine("Result of Async Call #2 : {0}", logId2);

				int logId3 = proxy.EndLogMessage(ar3);
				Console.WriteLine("Result of Async Call #3 : {0}", logId3);
			}
		}
	}
}
