﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WinClientApp.LogService;
using System.ServiceModel;

namespace WinClientApp
{
	public partial class ClientForm : Form
	{
		public ClientForm()
		{
			InitializeComponent();
			StopAnimation();
		}

		private void BeginAnimation()
		{
			progressBar1.Style = ProgressBarStyle.Marquee;
			progressBar1.MarqueeAnimationSpeed = 50;
			label1.Text = "Calling the LogService......";
			this.Update();
		}

		private void StopAnimation()
		{
			progressBar1.Style = ProgressBarStyle.Blocks;
			progressBar1.MarqueeAnimationSpeed = 0;
			label1.Text = "";
			this.Update();
		}

		private void btnSyncCall_Click(object sender, EventArgs e)
		{
			BeginAnimation();
			using (LogClient proxy = new LogClient("BasicHttpBinding_ILog")) {
				// 동기 호출
				int logId = proxy.LogMessage("Sync Call");
				MessageBox.Show("Result = " + logId.ToString(), "Result",
					MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
			StopAnimation();
		}

		private void btnAsyncCall_Click(object sender, EventArgs e)
		{
			BeginAnimation();
			using (LogClient proxy = new LogClient("BasicHttpBinding_ILog")) {
				// 비동기 호출
				IAsyncResult ar = proxy.BeginLogMessage("Async Call", null, null);

				//while (ar.IsCompleted == false) {
				//    Application.DoEvents();
				//}

				while (ar.AsyncWaitHandle.WaitOne(50) == false) {
					Application.DoEvents();
				}

				int logId = proxy.EndLogMessage(ar);
				// 결과 표시
				MessageBox.Show("Result = " + logId.ToString(), "Result",
					MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
			StopAnimation();
		}

		private void btnAsyncCall2_Click(object sender, EventArgs e)
		{
			//BeginAnimation();
			//LogClient proxy = new LogClient("BasicHttpBinding_ILog");
			//proxy.BeginLogMessage("Async Call", new AsyncCallback(OnCompelte), proxy);

			LogClient proxy = new LogClient("BasicHttpBinding_ILog");
			AsyncCallback callback = delegate(IAsyncResult ar) {
				try {
					int logId = proxy.EndLogMessage(ar);
					(proxy as IDisposable).Dispose();

					MethodInvoker func = delegate {
						StopAnimation();
						MessageBox.Show("Result = " + logId.ToString(), "Result",
							MessageBoxButtons.OK, MessageBoxIcon.Information);
					};
					if (this.InvokeRequired == true) {
						this.Invoke(func);
					}
					else {
						func();
					}
				}
				catch (Exception ex) {
					// 이 메시지 박스 호출도 쓰레드 마샬링을 해야만 한다.
					MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			};
			BeginAnimation();
			proxy.BeginLogMessage("Async call", callback, null);
		}

		private void OnCompelte(IAsyncResult ar)
		{
			try {
				LogClient proxy = (LogClient)ar.AsyncState;
				int logId = proxy.EndLogMessage(ar);
				(proxy as IDisposable).Dispose();

				MethodInvoker func = delegate {
					StopAnimation();
					MessageBox.Show("Result = " + logId.ToString(), "Result",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
				};
				if (this.InvokeRequired == true) {
					this.Invoke(func);
				}
				else {
					func();
				}
			}
			catch (Exception ex) {
				// 이 메시지 박스 호출도 쓰레드 마샬링을 해야만 한다.
				MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void btnExceptionTest_Click(object sender, EventArgs e)
		{
			BeginAnimation();
			using (LogClient proxy = new LogClient("BasicHttpBinding_ILog")) {

				IAsyncResult ar = proxy.BeginLogMessageWithError(null, null);

				while (ar.AsyncWaitHandle.WaitOne(50, false) == false) {
					Application.DoEvents();
				}

				try {
					proxy.EndLogMessageWithError(ar);
				}
				catch (FaultException ex) {
					MessageBox.Show(ex.Message, "Falut Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
			StopAnimation();
		}
	}
}
