﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;

namespace CallbackTestService
{
	[ServiceContract]
	interface ITestCallbackContract
	{
		[OperationContract]
		void OnCallback(string message);
	}

	[ServiceContract(CallbackContract = typeof(ITestCallbackContract))]
	interface ITestContract
	{
		[OperationContract]
		void TestMethod();
	}

	[ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, 
		ConcurrencyMode = ConcurrencyMode.Reentrant)]
	class TestService : ITestContract
	{
		public void TestMethod()
		{
			Console.WriteLine("TestMethod() called...");
			OperationContext ctx = OperationContext.Current;
			ITestCallbackContract callback = ctx.GetCallbackChannel<ITestCallbackContract>();
			callback.OnCallback("From TestService.TestMethod");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Simple Callback Test Service Sample......");

			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/callback/test"));
			WSDualHttpBinding binding = new WSDualHttpBinding();
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();

			binding.Security.Mode = WSDualHttpSecurityMode.None;
			behavior.HttpGetEnabled = true;

			host.AddServiceEndpoint(typeof(ITestContract), binding, "");
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press any key to shuthdown the service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
